// toShare co_hosts.js
'use strict';
// uses rest, tooltip

const sectionCoHosts = document.querySelector('section.co-hosts');
const buttonNew = sectionCoHosts.querySelector('button.new');
const buttonUp = sectionCoHosts.querySelector('button.up');
const buttonDown = sectionCoHosts.querySelector('button.down');
const tableCoHosts = sectionCoHosts.querySelector('table');
const tbodyCoHosts = sectionCoHosts.querySelector('tbody');
const trTemplateCoHosts = tbodyCoHosts.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputHostId = divEditLayer.querySelector('input.host-id');
const inputHostName = divEditLayer.querySelector('input.host-name');
const inputHostHost = divEditLayer.querySelector('input.host-host');
const inputHostPort = divEditLayer.querySelector('input.host-port');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');
const UNSELECT = -1;
let memoMouseOutside = false;
let orderStartIndex = UNSELECT;
let orderEndIndex = UNSELECT;

async function updateCoHostsList(){
	await atsCoHosts.restGet(WITH_URL);
	tbodyCoHosts.innerHTML = '';
	for(const atsACoHosts of atsCoHosts.data){
		const coHostsTr = trTemplateCoHosts.cloneNode(true);
		coHostsTr.dataset.hostId = atsACoHosts.host_id;
		const td = coHostsTr.querySelectorAll("td");
		td[0].innerText = atsACoHosts.host_name;
		td[1].innerText = atsACoHosts.host_host;
		td[2].innerText = atsACoHosts.host_port;

		td[0].addEventListener('click', onTdCoHostsClick);
		td[1].addEventListener('click', onTdCoHostsClick);
		td[2].addEventListener('click', onTdCoHostsClick);
		td[3].addEventListener('click', onTdOrderClick);
		tbodyCoHosts.appendChild(coHostsTr);
	}
}

function redrawSelected(){
	buttonUp.disabled = (orderStartIndex===UNSELECT)||(orderStartIndex===0);
	buttonDown.disabled = (orderEndIndex===UNSELECT)||(orderEndIndex===tbodyCoHosts.rows.length-1);
	for(const tbodyTr of tbodyCoHosts.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('select')){tbodyTr.classList.remove('select');}
	}
	if(orderStartIndex != UNSELECT){
		for(let index=orderStartIndex; index <= orderEndIndex; index++){
			tbodyCoHosts.rows[index].classList.add('select');
		}
	}
}


function onNewEdit(){
	for(const tbodyTr of tbodyCoHosts.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'flex';
	inputHostId.value = "0";
	inputHostName.value = "";
	inputHostHost.value = "";
	inputHostPort.value = "";
	inputHostName.focus();
	buttonAdd.disabled = true;
	buttonFix.disabled = true;
	buttonDel.disabled = true;
}

async function onOrderExecute(event){
	let hostIds = [];
	for(let index =orderStartIndex; index <= orderEndIndex; index++){
		hostIds.push(atsCoHosts.data[index].host_id);
	}
	switch(event.target){
		case buttonUp:{
			hostIds.push(atsCoHosts.data[orderStartIndex-1].host_id);
			try{
				await atsCoHosts.restOrderPut(hostIds)
				orderStartIndex -=1;
				orderEndIndex -=1;
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDown:{
			hostIds.unshift(atsCoHosts.data[orderEndIndex+1].host_id);
			try{
				await atsCoHosts.restOrderPut(hostIds)
				orderStartIndex +=1;
				orderEndIndex +=1;
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	await updateCoHostsList();
	redrawSelected();
}

function onTdCoHostsClick(event){
	orderStartIndex = UNSELECT;
	orderEndIndex = UNSELECT;
	redrawSelected();
	if(!event.currentTarget.parentElement.classList.contains('current')){
		for(const tbodyTr of tbodyCoHosts.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.target.parentElement.classList.add('current');
		divEditLayer.style.display = 'flex';
		const atsACoHosts = atsCoHosts.findField('host_id', Number(event.currentTarget.parentElement.dataset.hostId));
		inputHostId.value = atsACoHosts.host_id;
		inputHostName.value = atsACoHosts.host_name;
		inputHostHost.value = atsACoHosts.host_host;
		inputHostPort.value = atsACoHosts.host_port;
		inputHostName.select();
		inputHostName.focus();
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.parentElement.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}

function onTdOrderClick(event){
	let index = event.currentTarget.parentElement.rowIndex -1;
	if((orderStartIndex > index)||(orderStartIndex===UNSELECT)){
		orderStartIndex = index;
		orderEndIndex = index;
	}else if(orderStartIndex===index){
		orderStartIndex = UNSELECT;
		orderEndIndex = UNSELECT;
	}else if(orderStartIndex < index){
		orderEndIndex = index;
	};
	redrawSelected();
}

function onOutsideClick(event){
	if(event.currentTarget===event.target){
		if(memoMouseOutside){
			onCancel();
		}else{
			 memoMouseOutside = true;
		}
	}else{
		memoMouseOutside = false;
	}
}
function onCancel(){
	for(const tbodyTr of tbodyCoHosts.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}

function onCoHostsModified(){
	if(Number(inputHostId.value)===0){
		buttonAdd.disabled = (inputHostName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
async function onExecute(event){
	switch(event.target){
		case buttonAdd:{
			try{
				await atsCoHosts.restPost(
					inputHostName.value,
					inputHostHost.value,
					Number(inputHostPort.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsCoHosts.restPut(
					Number(inputHostId.value),
					inputHostName.value,
					inputHostHost.value,
					Number(inputHostPort.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsCoHosts.restDelete(
					Number(inputHostId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateCoHostsList();
}


tbodyCoHosts.innerHTML = '';
atsConfiguration.reflection();

updateCoHostsList();

divEditLayer.addEventListener('mousedown', onOutsideClick);
divEditLayer.addEventListener('mouseup', onOutsideClick);
buttonNew.addEventListener('click', onNewEdit);
buttonUp.addEventListener('click', onOrderExecute);
buttonDown.addEventListener('click', onOrderExecute);
buttonCancel.addEventListener('click', onCancel);
inputHostName.addEventListener('input', onCoHostsModified);
inputHostHost.addEventListener('input', onCoHostsModified);
inputHostPort.addEventListener('input', onCoHostsModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);

